classdef siMesh < handle 
  
  properties %(SetAccess = protected)
    d =0;
    dim=0;
    sTh={};
    nsTh=0;
    toGlobal=[];
    toParent=[];
%    nqGlobal=0;;
    sThsimp=[];
    sThlab=[];
    sThcolors=[];
    bbox=[]; % Bounding box
    sThgeolab=[];
    sThphyslab=[];
    sThpartlabs={};
    sThboundlabs={}; % 
    %sElts=[];
    % Ms=[];
    nq=0;
    nqParents = []; 
    toParents ={};
    other=[];
  end
  
  methods
  
    function self = siMesh(cFileName,varargin)
      if nargin==0, return;end
      if nargin>0
        assert(ischar(cFileName),'First parameter must be file name (char)')
        p = inputParser;
        p.addParamValue('dim', 2, @isfloat );
        p.addParamValue('d', [] );
        %p.addParamValue('perm',true); % permutation of local points : (false : as in (medit format))
        p.addParamValue('color',true);
        p.addParamValue('format','gmsh',@(x) ismember(x,{'medit','gmsh','freefem','triangle'}));
        p.addParamValue('isPhysLabel',true,@islogical);
        p.addParamValue('trans',[]);
        p.parse(varargin{:});
        R=p.Results;
        if isempty(R.d),  R.d=[R.dim:-1:(R.dim-1)];end
        switch R.format
          case('medit')
                  self=fc_simesh.read.readMedit(cFileName,R.dim,R.d);
          case('gmsh')
                  self=fc_simesh.read.readgmsh(cFileName,R.dim,R.d,R.isPhysLabel,R.trans);
          case('freefem')
                  self=fc_simesh.read.readfreefem(cFileName,R.dim,R.d);
          case('triangle')
                  self=fc_simesh.read.readtriangle(cFileName,R.dim);
        end
        if R.color, self.set_colors(); end
        self.nsTh=length(self.sTh);
        self.setTags();
        self.set_bbox();
        self.other.meshfile=cFileName;
        self.other.format=R.format;
        self.other.trans=R.trans;
      end
    end 
    
    function self=set_bbox(self)
      self.bbox=self.sTh{1}.bbox;
      for k=2:self.nsTh
        bb=self.sTh{k}.bbox;
        for i=1:self.dim
          self.bbox(2*i-1)=min(self.bbox(2*i-1),bb(2*i-1));
          self.bbox(2*i)=max(self.bbox(2*i),bb(2*i));
        end
      end
    end
    
    function self=set_colors(self)
      self.sThcolors=zeros(self.nsTh,3);
      for d=0:self.d
        Ilab=self.find(d);
        if ~isempty(Ilab)
          self.sThcolors(Ilab,:)=fc_tools.graphics.selectColors(length(Ilab));
        end
        for jlab=Ilab
          self.sTh{jlab}.color=self.sThcolors(jlab,:);
        end
      end
    end
    
    function labs=get_labels(self,d)
       labs=self.sThlab(self.find(d));
    end
    
    function self=setTags(self)
      for i=1:self.nsTh
        Tag=sprintf('d:%d, label : %d\nnq=%d, nme=%d',self.sTh{i}.d,self.sTh{i}.label,self.sTh{i}.nq,self.sTh{i}.nme);
        self.sTh{i}.Tag=Tag;
      end
    end
    
    
    function self=set_d(self)
      self.d=max(self.sThsimp);
    end
    
    function self=set_toGlobal(self)
      R=cellfun(@(x) x.toGlobal,self.sTh,'UniformOutput',false);
      self.toGlobal=unique([R{:}]);
    end
    
    function self=set_Parents(self)
      for i=1:self.nsTh
        [C,ia,ib]=intersect(self.sTh{i}.toGlobal,self.toGlobal);
        self.sTh{i}.toParent=ib'; % to delete in future version
        self.sTh{i}.toParents{1}=ib';
        self.sTh{i}.nqParent=self.nq; % to delete in future version
        self.sTh{i}.nqParents(1)=self.nq;
      end
    end
    
    function toG=get_toGlobal(self,d,labels)
      idx=self.find(d,labels);
      C=cellfun(@(x) x.toGlobal,{self.sTh{idx}},'UniformOutput',false);
      toG=unique([C{:}]);
    end
    
    function self=AddCopysiMeshElt(self,p1Elmt)
      I=intersect(p1Elmt.toGlobal,self.toGlobal); % On verifie que le p1Elmt appartient au P1Mesh
      if length(I)==length(p1Elmt.toGlobal)
        
        N=self.nsTh+1;
        self.nsTh=N;
        self.sTh{N}=p1Elmt.copy();
        
        self.sThsimp(N)=p1Elmt.d;
        self.sThlab(N)=p1Elmt.label;
      end
    end    
    
    function Th=copy(self)
      Th=fc_simesh.siMesh();
      Th.d=self.d;
      Th.dim=self.dim;
      Th.nsTh=self.nsTh;
      Th.sTh=cell(1,Th.nsTh);
      for i=1:Th.nsTh, Th.sTh{i}=self.sTh{i}.copy();end
      Th.toGlobal=self.toGlobal;
      Th.toParent=self.toParent;
      Th.sThsimp=self.sThsimp;
      Th.sThlab=self.sThlab;
      Th.sThcolors=self.sThcolors;
      Th.bbox=self.bbox;
      Th.sThgeolab=self.sThgeolab;
      Th.sThphyslab=self.sThphyslab;
      Th.sThboundlabs=self.sThboundlabs;
      Th.nq=self.nq;
      Th.nqParents=self.nqParents;
      Th.toParents=self.toParents;
      Th.other=self.other;
    end
    
    function self=AddsiMeshElt(self,ooElmt)
      if ~isempty(self.sTh)
        I=intersect(ooElmt.toGlobal,self.toGlobal); % On verifie que le ooElmt appartient au ooMesh
        assert( length(I)==length(ooElmt.toGlobal) );
        N=self.nsTh+1;
      else
        N=1;
        self.toGlobal=ooElmt.toGlobal;
        self.nq=ooElmt.nq;
      end
        self.nsTh=N;
        self.sTh{N}=ooElmt;
        self.sThsimp(N)=ooElmt.d;
        self.sThlab(N)=ooElmt.label;
    end
    
    function self=AddsiMeshElts(self,d,q,nme,me,mel,varargin)
      p = inputParser;
      p.addParamValue('geolab', [] );
      p.addParamValue('partlab', {} );
      p.parse(varargin{:});
      R=p.Results;
      labels=unique(mel);
      k=self.nsTh;
      nq=size(q,2);
      partlab={};
      geolab=[];
      if d==self.d
        %self.toGlobal=unique([self.toGlobal,unique(me)']);
        self.toParent=unique([self.toParent,unique(me)']);
        self.toGlobal=self.toParent;
        self.toParents{1}=self.toParent;
        self.nqParents(1)=size(q,2);
      end
      for i=1:length(labels)
        I=find(mel==labels(i));
        ME=me(:,I);
        if ~isempty(R.geolab), geolab=R.geolab(I);end
        if ~isempty(R.partlab), partlab=R.partlab(I);end
        indQ=unique(ME);
        Q=q(:,indQ);
        lQ=1:length(indQ);
        J=zeros(nq,1);
        J(indQ)=lQ;
        if(d==0) %correctif a faire si d=0 (programmation a l'arrache! To improve)
          MELoc=J(ME)';
        else
          MELoc=J(ME);
          end
        self.sTh{k+i}=fc_simesh.siMeshElt(self.dim,d,labels(i),Q,MELoc,indQ',nq,geolab,partlab);
        self.sThsimp(k+i)=d;
        self.sThlab(k+i)=labels(i);
      end
      self.nsTh=self.nsTh+length(labels);
    end    
 
    function Ba=barycenters(self,varargin)
      % Barycenters of selected elements
      p = inputParser; 
      p.addParamValue('d',self.d,@isscalar);
      p.addParamValue('labels',[]); % select labels to draw
      p.parse(varargin{:});
      R=p.Results;
      if isempty(R.labels), idxlab=self.find(R.d);else idxlab=self.find(R.d,R.labels);end
      if isempty(idxlab), return;end
      j=1;
      Ba=zeros(self.dim,length(idxlab));
      for ilab=[idxlab]
        Ba(:,j)=mean(self.sTh{ilab}.barycenters(),2);
        j=j+1;
      end
    end
 
    function obj=DeletesiMeshElt(self,idx)
      % idx : partition label
      obj=siMesh();
      obj.d=self.d;
      obj.dim=self.dim;
      %obj.nsTh=self.nsTh-1;
      obj.toGlobal=self.toGlobal;
      I=setdiff(1:self.nsTh,idx);
      for i=[I]
        obj=obj.AddCopysiMeshElt(self.sTh{i});
      end
      obj.set_colors();
      obj.setTags();
      obj.set_bbox();
      
      I=[];for i=1:obj.nsTh,I=[i,obj.sTh{i}.toGlobal];end
      [toGlobal,ia,ic]=unique(I);
      obj.nq=length(toGlobal);
      obj.toGlobal=1:obj.nq;
      J=1:obj.nq; 
      B=sparse(toGlobal,ones(1,obj.nq),J);
      for i=1:obj.nsTh
        obj.sTh{i}.toGlobal=full(B(obj.sTh{i}.toGlobal,1))';
        obj.sTh{i}.nqGlobal=obj.nq;
        obj.sTh{i}.toParent=obj.sTh{i}.toGlobal;
        obj.sTh{i}.nqParent=obj.nq;
      end
    end 
    

    function disp(self)
    % FUNCTION disp, method function of the siMesh class.
    %   Displays informations on the siMesh object.
    %
    % Copyright (C) 2017-2010 F. Cuvelier
      fc_tools.utils.disp_object(self);
    end    
    
    function u=eval(self,fun,varargin)
    % Eval fun on siMesh object and return an array with shape (self.nq,1)
    %        corresponding on the evaluate of fun on each mesh vertices (if fun is not a 
    %        list and not None)
    %        
    %        fun could be:
    %          - a scalar
    %          - a function handle
    %          - an array 
    %          - a cell array where each element could be on of the previous type
      if isnumeric(fun)
        if size(fun,1)==self.nq && size(fun,2)==1
          u=fun;
          return
        end
        if size(fun,2)==self.nq && size(fun,1)==1
          u=fun.';
          return
        end
      end
      
      p = inputParser; 
      p.addParamValue('level',0);
      p.addParamValue('d',self.d,@isscalar);
      p.addParamValue('labels',[]);
      p.addParamValue('cell',false,@islogical);
      p.parse(varargin{:});
      R=p.Results;
      
      if iscell(fun)
        assert(size(fun,1)==1,'fun must be an 1-by-m cell array, given %d-by-%d ',size(fun))
        m=length(fun);
        if R.cell
          u=cell(m,1);
          for i=1:m
            u{i}=self.eval(fun{i},varargin{:});
          end
        else
          u=zeros(self.nq,m);
          for i=1:m
            u(:,i)=self.eval(fun{i},varargin{:});
          end
        end
        return
      end
      
      assert(ismember(R.level,0:length(self.nqParents)))
      
      if isempty(R.labels), idxlab=self.find(self.d);else idxlab=self.find(R.d,R.labels);end
      
      if R.level==0
        u=zeros(self.nq,1);
      else
        u=zeros(self.nqParents(R.level),1);
      end
      for i=idxlab
        u(self.sTh{i}.toParents{R.level+1})=self.sTh{i}.eval(fun);
      end  
    end    
    
    function obj=extractPartition(self,plabel)
      % label : partition label
      i=self.find(self.d,plabel);
      if isempty(i), obj=[];return; end
      obj=siMesh();
      obj.d=self.d;
      obj.dim=self.dim;
      l=1; 
      obj.sTh{l}=self.sTh{i};
      obj.nsTh=1;
      obj.sThsimp(l)=obj.d;
      obj.sThlab(l)=plabel;
      q=obj.sTh{l}.q;
      nq=obj.sTh{l}.nq;
      obj.nq=obj.sTh{l}.nq;
      I=self.find(self.d-1);
      for k=1:length(I)
        i=I(k);
        if self.sTh{i}.label>0
          if sum(cellfun(@(x) size(x,1),self.sTh{i}.partlab)) == length(self.sTh{i}.partlab)
            partlab=cellfun(@(x) x(1),self.sTh{i}.partlab);
            P=find(partlab==label);
            if ~isempty(P)
              me=self.sTh{i}.me(:,P);
              mel=self.sTh{i}.label*int32(ones(1,length(P)));
              obj.AddElements(self.d-1,self.sTh{i}.q,length(mel),me,mel,self.sTh{i}.indve);
            end
          end
        else % fictif (ghost)
          Ql=self.sTh{i}.q(:,self.sTh{i}.me(:,1));
          % Check if all Ql points are in q? Un peu pourri comme methode! => a ameliorer
          isIn=true;r=1;nQl=size(Ql,2);
          while ((isIn)&&(r<=nQl))
            S=sqrt(sum((q-Ql(:,r)*ones(1,nq)).^2,1));
            A=find(S<1e-6);
            if isempty(A), isIn=false;end
            r=r+1;
          end
          if isIn
            obj.nsTh=obj.nsTh+1;
            obj.sTh{obj.nsTh}=self.sTh{i};
            obj.sThsimp(obj.nsTh)=self.d-1;
            obj.sThlab(obj.nsTh)=self.sTh{i}.label;
          end
        end
      end
    end    
 
    function obj=extractSubDomain(self,label)
      % label : partition label
      i=self.find(self.d,label);
      if isempty(i), obj=[];return; end
      obj=PkMesh();
      obj.d=self.d;
      obj.dim=self.dim;
      obj.order=self.order;
      l=1; 
      obj.sTh{l}=self.sTh{i};
      obj.nsTh=1;
      obj.sThsimp(l)=obj.d;
      obj.sThlab(l)=label;
      q=obj.sTh{l}.q;
      nq=obj.sTh{l}.nq;
      obj.nq=obj.sTh{l}.nq;
      I=self.find(self.d-1);
      for k=1:length(I)
        i=I(k);
        if self.sTh{i}.label>0
          if sum(cellfun(@(x) size(x,1),self.sTh{i}.partlab)) == length(self.sTh{i}.partlab)
            partlab=cellfun(@(x) x(1),self.sTh{i}.partlab);
            P=find(partlab==label);
            if ~isempty(P)
              me=self.sTh{i}.me(:,P);
              mel=self.sTh{i}.label*int32(ones(1,length(P)));
              obj.AddElements(self.d-1,self.sTh{i}.q,length(mel),me,mel,self.sTh{i}.indve);
            end
          end
        else % fictif (ghost)
          Ql=self.sTh{i}.q(:,self.sTh{i}.me(:,1));
          % Check if all Ql points are in q? Un peu pourri comme methode! => a ameliorer
          isIn=true;r=1;nQl=size(Ql,2);
          while ((isIn)&&(r<=nQl))
            S=sqrt(sum((q-Ql(:,r)*ones(1,nq)).^2,1));
            A=find(S<1e-6);
            if isempty(A), isIn=false;end
            r=r+1;
          end
          if isIn
            obj.nsTh=obj.nsTh+1;
            obj.sTh{obj.nsTh}=self.sTh{i};
            obj.sThsimp(obj.nsTh)=self.d-1;
            obj.sThlab(obj.nsTh)=self.sTh{i}.label;
          end
        end
      end
    end
    
    function u=feval(self,fun,varargin)
    % Eval fun on siMesh object and return an array with shape (self.nq,1)
    %        corresponding on the evaluate of fun on each mesh vertices (if fun is not a 
    %        list and not None)
    %        
    %        fun could be:
    %          - a function handle
    %          - a cell array of function handles

      if iscell(fun)
        assert(size(fun,1)==1,'fun must be an 1-by-m cell array, given %d-by-%d ',size(fun))
        m=length(fun);
        u=zeros(self.nq,m);
        for i=1:m
          assert(fc_tools.utils.isfunhandle(fun{i}),'%d-th cell must be a function handle',i)
          u(:,i)=self.feval(fun{i},varargin{:});
        end
        return 
      end
      
      assert(fc_tools.utils.isfunhandle(fun))
      assert( ismember(nargin(fun),[1,self.dim]) ,'Number of function arguments must be 1 or %d',self.dim)

      p = inputParser; 
      p.addParamValue('d',self.d,@isscalar);
      p.addParamValue('level',0,@isscalar); % mainly used with partition meshes?
      p.addParamValue('labels',[]);
      p.parse(varargin{:});
      R=p.Results;
      assert(ismember(R.level,0:length(self.nqParents)))
      
      if isempty(R.labels), idxlab=self.find(self.d);else idxlab=self.find(R.d,labels);end
      
      if R.level==0
        u=zeros(self.nq,1);
      else
        u=zeros(self.nqParents(R.level),1);
      end
      for i=idxlab
        u(self.sTh{i}.toParents{R.level+1})=self.sTh{i}.feval(fun);
      end
    end    
    
    function ind=find(this,varargin)
      switch nargin
        case 2
          % d=varargin{1}
          ind=find(this.sThsimp==varargin{1});
        case 3
          %ind=find((this.sThsimp==varargin{1}) & (this.sThlab==varargin{2}));
          I=find(this.sThsimp==varargin{1});
          [J,c]=intersect(this.sThlab(I),varargin{2}); % Add test if labels are OK
          ind=I(c);
    %        if length(ind)<length(varargin{2})
    %          str=sprintf(' %d,',setdiff(varargin{2},this.sThlab(I)));str(end)='.';
    %          warning('labels not found for %d-simplex sub-meshes : %s',varargin{1},str);
    %        end
        otherwise
          ind=[];
          warning('unknown number of arguments')
      end 
      %if isempty(ind), warning('Sub mesh not found ');end
    end    
    
    function labName=funLabName(self,d)
      if self.d==d
        if fc_tools.comp.isOctave()
          labName=@(i) ['\Omega_{',num2str(i),'}'];
        else
          %labName=@(i) ['$\Omega_{',num2str(i),'}$'];
          labName=@(i) ['\Omega_{',num2str(i),'}'];
        end
        return;
      elseif self.d==(d+1)
      if fc_tools.comp.isOctave()
          labName=@(i) ['\Gamma_{',num2str(i),'}'];
        else
          %labName=@(i) ['$\Gamma_{',num2str(i),'}$'];
          labName=@(i) ['\Gamma_{',num2str(i),'}'];
        end
        return;
      elseif self.d==(d+2)
        if fc_tools.comp.isOctave()
          labName=@(i) ['\partial\Gamma_{',num2str(i),'}'];
        else
          %labName=@(i) ['$\partial\Gamma_{',num2str(i),'}$'];
          labName=@(i) ['\partial\Gamma_{',num2str(i),'}'];
        end
        return
      elseif self.d==(d+3)
        if fc_tools.comp.isOctave()
          labName=@(i) ['\partial^2\Gamma_{',num2str(i),'}'];
        else
          %labName=@(i) ['$\partial\Gamma_{',num2str(i),'}$'];
          labName=@(i) ['\partial^2\Gamma_{',num2str(i),'}'];
        end
        return
      end
      labName=[];
      return
    end    
    
    function cl=get_cl(self)
    % get characteristic length
      i=1:self.dim;
      cl=max(self.bbox(2*i)-self.bbox(2*i-1));
    end
    
    function h=get_h(self)
      h=0;
      for i=1:self.nsTh
        h=max(h,self.sTh{i}.h);
      end
    end    

    function [q,me,toglobal]=get_mesh(self,varargin)
      p = inputParser;
      p.addParamValue('d',self.d);
      p.addParamValue('labels',[]);
      p.parse(varargin{:});
      R=p.Results;
      if isempty(R.labels), idxlab=self.find(R.d);else idxlab=self.find(R.d,R.labels);end
      
      me=[];q=[];toglobal=[];
      if isempty(idxlab)
        if ~isempty(R.labels)
          warning('No labels index found with d=%d and labels=%s',R.d,num2str(R.labels))
        else
          warning('No labels index found with d=%d',R.d)
        end
        return
      end
      
      toglobal_loc=[];
      me=[];
      for ilab=[idxlab]
        sTh=self.sTh{ilab};
        if size(sTh.toParents{1},2)==1 % rustine pour Octave
          toglobal_loc=[toglobal_loc,sTh.toParents{1}'];
        else
          toglobal_loc=[toglobal_loc,sTh.toParents{1}];
        end
        if isempty(me)
          me=reshape(sTh.toParents{1}(sTh.me),size(sTh.me)); % reshape needed when only one element
        else
          me=[me,reshape(sTh.toParents{1}(sTh.me),size(sTh.me))]; % reshape needed when only one element
        end
        %else
        %  me=[me,sTh.toParents{1}(sTh.me)];
        %end
      end
      [toglobal,IR,IL] = unique(toglobal_loc); % toglobal(IL)==toglobal_loc and toglobal==toglobal_loc(IR)
      nq=length(toglobal);
      idx=zeros(1,self.nq);
      idx(toglobal)=1:nq;
      q=zeros(self.dim,nq);
      for ilab=[idxlab]
        sTh=self.sTh{ilab};
        q(:,idx(sTh.toParents{1}))=sTh.q;
      end
      me=idx(me);
      
      if size(me,1)~=R.d+1, me=me';end
    end    
    
    function nme=get_nme(self,varargin)
      p = inputParser;
      p.addParamValue('d',self.d);
      p.addParamValue('labels',[]);
      p.parse(varargin{:});
      R=p.Results;
      if isempty(R.labels), idxlab=self.find(R.d);else idxlab=self.find(R.d,R.labels);end
      nme=0;
      for ilab=[idxlab]
        nme=nme+self.sTh{ilab}.nme;
      end
    end 
    
    function nq=get_nq(self,varargin)
      p = inputParser;
      p.addParamValue('d',self.d);
      p.addParamValue('labels',[]);
      p.parse(varargin{:});
      R=p.Results;
      if isempty(R.labels) && R.d==self.d, nq=self.nq; return;end
      if isempty(R.labels), idxlab=self.find(R.d);else idxlab=self.find(R.d,R.labels);end
      if length(idxlab)==0, nq=0;return;end
      if length(idxlab)==1, nq=self.sTh{ilab}.nq;return;end
      
      me=[];q=[];toglobal=[]; 
      toglobal_loc=[];
      me=[];
      for ilab=[idxlab]
        sTh=self.sTh{ilab};
        toglobal_loc=[toglobal_loc,sTh.toParents{1}];
      end
      toglobal= unique(toglobal_loc); % toglobal(IL)==toglobal_loc and toglobal==toglobal_loc(IR)
      nq=length(toglobal);
    end    
    
    function info(self,varargin)
      % version simplifiée pour le moment
      p = inputParser; 
      p.addParamValue('variable',true,@islogical);
      p.addParamValue('verbose',0,@(x) ismember(x,0:1));
      p.parse(varargin{:});
      R=p.Results;
      if R.variable
        fprintf('Variable %s [%s object] :\n',inputname(1),class(self));
      end
      if R.verbose>0, fprintf('  dim=%d, d=%d\n',self.dim,self.d);end
      fprintf('  nq=%d, nme=%d\n',self.nq,self.get_nme());
      if R.verbose>0
        for d=self.d:-1:0
          I=self.find(d);
          if  isempty(I)
            fprintf('  %d-simplices : none\n',d);
          else
            fprintf('  %d-simplices : number %d, labels : ',d,length(I));
            fprintf('%d ',self.sThlab(I));fprintf('\n')
          end
        end
      end
    end    
    
    function move(Th,u)
      % Move mesh
      % version simplifiée pour le moment
      assert(isnumeric(u),'Needs a %d-by-%d or %d-by-%d  array for the vector displacement',Th.dim,Th.nq,Th.nq,Th.dim)
      assert( all(size(u)==[Th.dim,Th.nq]) || all(size(u)==[Th.nq,Th.dim]) ,'Mismatched dimensions of vector displacement array')
      if size(u,1)==Th.dim
        extract=@(I) u(:,I);  % to obtain a dim-by-N array
      else
        extract=@(I) u(I,:).'; % to obtain a dim-by-N array
      end
%        elseif iscell(u)
%          assert( length(u)==Th.dim, 'Mismatched dimensions of cell vector displacement')
%          extract=@(I) u(I,:);
%      end
      for i=1:Th.nsTh
        Th.sTh{i}.move(extract(Th.sTh{i}.toGlobal));
      end
      
    end    
    
    function obj=set_nq(obj)
      idxdom=obj.find(obj.d);
      if length(idxdom)==1
        obj.nq=obj.sTh{idxdom}.nq;
      elseif length(idxdom)>1
        toG=[];
        for idx=[idxdom]
          toG=[toG,obj.sTh{idx}.toGlobal];
        end
        obj.nq=length(unique(toG));
      else
        error('Unable to set nq')
      end
    end    
    
    function varargout=vtkWrite(Th,FileName,varargin)
      p = inputParser;
      p.addParamValue('Values',{},@(x) iscell(x) || isnumeric(x) );
      p.addParamValue('Names',{},@(x) (iscell(x) || ischar(x)));
      p.addParamValue('idxsTh',[],@isnumeric);
      p.parse(varargin{:});
      U=p.Results.Values;
      names=p.Results.Names;
      idxsTh=p.Results.idxsTh;
      if isempty(idxsTh)
        idxsTh=Th.find(Th.d);
      else
        assert(isempty(setdiff(idxsTh,1:Th.nsTh)))
      end
      
      [pathstr,fname,fext] = fileparts(FileName);
      if exist([pathstr,filesep,fname])~=7,  mkdir([pathstr,filesep,fname]);end
      pvdfile=[pathstr,filesep,fname,'.pvd'];
      fid=fopen(pvdfile,'w');
      fprintf(fid,'<?xml version="1.0"?>\n');
      fprintf(fid,'<VTKFile type="Collection" version="1.0" byte_order="LittleEndian" header_type="UInt64">\n');
      fprintf(fid,'\t<Collection>\n');
      vtufile=[pathstr,filesep,fname,filesep,fname];
      svtufile=[fname,filesep,fname];
      idxfile=0;
      for i=[idxsTh]
        fullvtufile=sprintf('%s_%d.vtu',vtufile,idxfile);
        smallvtufile=sprintf('%s_%d.vtu',svtufile,idxfile);
        Th.sTh{i}.vtuWrite(fullvtufile,varargin{:});
        fprintf(fid,'\t\t<DataSet part="0"  file="%s"/>\n',smallvtufile);
        idxfile=idxfile+1;
      end
      
      fprintf(fid,'\t</Collection>\n');
      fprintf(fid,'</VTKFile>');
      fclose(fid);
      fprintf(' Paraview file %s created\n',pvdfile);
      if nargout==1, varargout{1}=pvdfile;end
    end    
      
    
    
%%% GRAPHICS    
    function varargout=plotmesh(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotmesh(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=plot(self,u,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plot(self,u,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=scatter(self,u,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.scatter(self,u,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=plotiso(self,u,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotiso(self,u,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=plotquiver(self,V,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotquiver(self,V,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=slicemesh(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.slicemesh(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=slice(self,u,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.slice(self,u,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=sliceiso(self,u,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.sliceiso(self,u,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
%      function varargout=plotGradBaCo(self,varargin)
%        if fc_tools.utils.is_fcPackage('siplt')
%          varargout=cell(1,nargout);
%          [varargout{:}]=fc_siplt.plotGradBaCo(self,varargin{:});
%        else
%          if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
%          warning('Unable to load fc_siplt %s',name)
%        end
%      end
    
    function varargout=plotElementsNumber(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotElementsNumber(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=plotVertices(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotVertices(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=plotVerticesNumber(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotVerticesNumber(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    
  end % methods 
end % classdef
