fc_tools.utils.cleaning()
N=30;
[bvp,info]=fc_vfemp1.examples.setBVPStationaryConvectionDiffusion2D01(N,2);
fc_vfemp1.examples.print_info(info)

fprintf('*** Solving %s\n',info.name)
[u,SolveInfo]=bvp.solve('time',true); 
fprintf('    -> ndof (number of degrees of freedom) = %d\n',length(u));
fc_vfemp1.examples.print_info(SolveInfo)

if fc_tools.utils.is_fcPackage('siplt')
  fprintf('*** Graphics with fc_siplt package\n');
  fc_tools.graphics.monitors.onGrid(3,3,'figures',1:7,'covers',0.9);
  interpreter=@(x) fc_tools.graphics.interpreter(x);
  if fc_tools.comp.isOctave(),opt_interp={'interpreter','tex'};else,opt_interp={'interpreter','latex'};end
  isOctave=fc_tools.comp.isOctave();
  
  tstart=tic();

  Th=bvp.Th;
  figure(1)
  Th.plotmesh('inlegend',true)
  hold on
  axis off,axis equal;
  legend('show')
  Th.plotmesh('d',1,'Linewidth',2,'Color','k')
  title('BVPStationaryConvectionDiffusion2D01')
  
  figure(2)
  Th.plotmesh('Color',0.85*[1,1,1])
  hold on,axis off,axis image;
  Th.plotmesh('d',1,'Linewidth',2,'inlegend',true)
  legend('show');
  title('BVPStationaryConvectionDiffusion2D01')

  figure(3)
  Th.plot(u)
  axis off;axis image,shading interp
  title(interpreter(sprintf('Numerical solution ( $n_q=%d$, $n_{me}=%d$ )',Th.nq,Th.get_nme())),opt_interp{:});
  h=colorbar();
  set(get(h,'title'),'string',interpreter('$u$'),opt_interp{:})
  
  figure(4)
  Th.plot(u)
  hold on,axis off,axis image
  Th.plotiso(u,'isorange',0,'Color','w','Linewidth',1.5);
  shading interp
  title(interpreter(sprintf('Numerical solution ( $n_q=%d$, $n_{me}=%d$ )',Th.nq,Th.get_nme())),opt_interp{:});
  h=colorbar();
  set(get(h,'title'),'string',interpreter('$u$'),opt_interp{:})
  
  figure(5)
  Th.plot(u,'FaceAlpha',0.8);
  hold on,axis off,axis image
  if isOctave % FaceAlpha not implemented in Octave
    Th.plotiso(u,'niso',20,'Linewidth',1.5,'color','w');
    colorbar()
  else
    [~,~,cax]=Th.plotiso(u,'niso',20,'Linewidth',1.5,'isocolorbar',true,'format','%.3f');
    %set(cax,options.colorbar{:})
  end
  shading interp
  title(interpreter(sprintf('Numerical solution ( $n_q=%d$, $n_{me}=%d$ )',Th.nq,Th.get_nme())),opt_interp{:});

  figure(6)
  Th.plot(u,'FaceColor','interp','FaceAlpha',0.8,'EdgeColor','none');
  hold on
  if isOctave
    Th.plotiso(u,'niso',20,'Linewidth',1.5,'color','w');
    colorbar()
  else
    [~,~,cax]=Th.plotiso(u,'niso',20,'Linewidth',1.5,'isocolorbar',true,'format','%.3f');
    %set(cax,options.colorbar{:})
  end
  Th.plotiso(u,'isorange',0,'Color','k','Linewidth',1.5);
  view(106,27)
  xlabel('x'),ylabel('y'),zlabel('z')
  
  figure(7)
  clf
  V=Th.eval(info.V);
  Th.plotquiver(V,'colordata',u,'scale',0.005,'freq',1);
  hold on,axis image, axis off
  colorbar()
  Th.plotmesh('d',1,'Color','k','Linewidth',2)
  tcpu=toc(tstart);
  fprintf('    -> Done it in %.3f(s)\n',tcpu)
  
end
